package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.model.TaDevice;
import com.huiju.welcome.model.TaPerson;
import com.huiju.welcome.service.ICameraService;
import com.huiju.welcome.service.ITaDeviceService;
import com.huiju.welcome.utils.Encrypt;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.TencentFaceUtils;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class CameraServiceImpl implements ICameraService {
    @Autowired
    private ITaDeviceService taDeviceService;

    @Autowired
    TencentFaceUtils tencentFaceUtils;

    @Override
    public String dispatchPerson(TaPerson taPerson, TaDevice... devices) {
        List<TaDevice> taDeviceList = null;
        if (null == devices || devices.length == 0) {
            QueryWrapper<TaDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("status",StatusUtils.Normal);
            queryWrapper.eq("afr",StatusUtils.BoolTrue);
            taDeviceList = taDeviceService.list(queryWrapper);
        } else {
            taDeviceList = Arrays.asList(devices);
        }

        //分发人脸到各个相机
        for(TaDevice device: taDeviceList) {
            try {
                Map<String, Object> payload = new HashMap<>();
                payload.put("personId", taPerson.getPersonId());
                payload.put("image", taPerson.getAvatar());

                Action act = Action.create(Action.TYPE_DISPATCH_FACE, payload);

                WebSocketServer.sendInfo(act.toJSON(), "device", device.getDeviceId());
            } catch (Exception e){
                e.printStackTrace();
                return "连接设备(" + device.getDeviceId() + ")人脸下发服务 失败: "+ e.getMessage();
            }
        }

        // 下发到腾讯云人脸识别
        try {
            tencentFaceUtils.createFace(String.valueOf(taPerson.getRealId()), taPerson.getAvatar());
        } catch (TencentCloudSDKException e) {
            e.printStackTrace();
            log.error("下发人脸到腾讯云失败: " + e.getMessage());
        }

        return "";
    }
}
