package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.SysUserMapper;
import com.huiju.welcome.mapper.TaMainUsherRecordMapper;
import com.huiju.welcome.mapper.TaVisitorAppointmentMapper;
import com.huiju.welcome.mapper.TaWechatUserMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.TaCustomerMapper;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaMainUsherRecord;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaMainUsherRecordService;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.MiniApp;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@Service
public class TaMainUsherRecordServiceImpl extends ServiceImpl<TaMainUsherRecordMapper, TaMainUsherRecord> implements ITaMainUsherRecordService {
    @Autowired
    private TaMainUsherRecordMapper taMainUsherRecordMapper;

    @Autowired
    private TaCustomerMapper taCustomerMapper;

    @Autowired
    TaVisitorAppointmentMapper taVisitorAppointmentMapper;

    @Autowired
    TaWechatUserMapper taWechatUserMapper;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    ITaCustomerService iTaCustomerService;

    @Autowired
    MiniApp miniApp;

    @Override
    public boolean updateNormal(List<TaMainUsherRecord> records) {
        if (records == null || records.size() == 0) return true;

        List<Integer> recs = new ArrayList<>();
        for (TaMainUsherRecord rec: records) {
            recs.add(rec.getRecId());
        }

        UpdateWrapper<TaMainUsherRecord> wrapper = new UpdateWrapper<>();
        wrapper.set("status", StatusUtils.Normal);
        wrapper.in("rec_id", recs);

        int row = taMainUsherRecordMapper.update(new TaMainUsherRecord(), wrapper);
        return row >= 0;
    }

    @Override
    public boolean newRecordByDevice(TaCustomer taCustomer, TaPerson taPerson) {
        TaMainUsherRecord taMainUsherRecord = newRecordByCust(taCustomer);
        taMainUsherRecord.setStatus(StatusUtils.Ready);
        taMainUsherRecord.setPersonId(taPerson.getRealId());
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        try {
            Action act = Action.create(Action.TYPE_NEW_MAIN_USHER, JSONObject.toJSONString(taMainUsherRecord, SerializerFeature.WriteMapNullValue));
            WebSocketServer.sendInfo(act.toJSON(), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {e.printStackTrace();}

        // 通知对应的置业
        if (null != taCustomer.getConsultantId()) {
            try {
                notifyConsultant(taMainUsherRecord, taCustomer);
            } catch (Exception e) {e.printStackTrace();}
        }

        return row > 0;
    }

    @Override
    public boolean newRecordByConsultant(TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = newRecordByCust(taCustomer);
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        return row > 0;
    }

    private TaMainUsherRecord newRecordByCust(TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = new TaMainUsherRecord();
        taMainUsherRecord.setCustomerId(taCustomer.getCustomerId());
        taMainUsherRecord.setPersonName(taCustomer.getCustomerName());
        taMainUsherRecord.setAvatar(taCustomer.getAvatar());
        taMainUsherRecord.setPhone(taCustomer.getPhone());
        taMainUsherRecord.setAppointment(taCustomer.getApointment());
        taMainUsherRecord.setConsultantId(taCustomer.getConsultantId());
        taMainUsherRecord.setChannel(taCustomer.getChannel());
        taMainUsherRecord.setPurpose(taCustomer.getPurpose());
        taMainUsherRecord.setPlateNumber(taCustomer.getPlateNumber());
        taMainUsherRecord.setVisiteDate(LocalDateTime.now());

        if (StringUtils.isEmpty(taCustomer.getPhone())) {
            taMainUsherRecord.setStatus(StatusUtils.Ready);
        } else {
            taMainUsherRecord.setStatus(StatusUtils.Normal);
        }

        return taMainUsherRecord;
    }

    @Override
    public IPage<TaMainUsherRecord> taMainUsherRecordList(IPage<TaMainUsherRecord> pg,String phone,String platNumber,String personName,Integer appointment) {
        IPage<TaMainUsherRecord>  taMainUsherRecordList= taMainUsherRecordMapper.taMainUsherRecordList(pg,phone,platNumber,personName,appointment);
        return taMainUsherRecordList;
    }

    @Override
    public void notifyConsultant(TaMainUsherRecord taMainUsherRecord, TaCustomer taCustomer) throws Exception {
        Integer realConsultant = taCustomer.getConsultantId();
        Integer proxyConsultant = taMainUsherRecord.getConsultantId();

        // 查找置业微信ID
        String openid = taWechatUserMapper.getOpenidOf(proxyConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
        if (StringUtils.isEmpty(openid)) {
            throw new Exception("置业尚未绑定微信小程序");
        }

        String visitorName = taMainUsherRecord.getPersonName();
        String phone = taMainUsherRecord.getPhone();
        Integer customerId = taMainUsherRecord.getCustomerId();

        String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + String.valueOf(customerId);
        String tplId = miniApp.getTpls().getVisitor();

        // 如果是客户对应的置业进行接待
        if (realConsultant.equals(proxyConsultant)) {
            List<Object> data = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add("您有新客户来访, 请及时接待!");
            }};

            miniApp.sendTPLMessage(tplId, openid, page, data);
        } else {
            // 否则, 给原置业发消息, 给接待置业发消息

            // 原绑定置业消息
            String rcOpenid = taWechatUserMapper.getOpenidOf(realConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(openid)) {
                throw new Exception("置业尚未绑定微信小程序");
            }

            SysUser usr1 = sysUserMapper.selectById(proxyConsultant);
            List<Object> dt1 = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add("您有新客户来访, 已被安排给 " + usr1.getUsername() + " 接待!");
            }};
            miniApp.sendTPLMessage(tplId, rcOpenid, page, dt1);



            // 代理置业
            SysUser usr2 = sysUserMapper.selectById(realConsultant);
            List<Object> dt2 = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add(usr2.getUsername() + " 有新客户来访, 请协助接待一下!");
            }};
            miniApp.sendTPLMessage(tplId, openid, page, dt2);
        }
    }
    public ResponseBean customerPage(int pageNum, int pageSize, String customerName, String phone, String platNumber) {
        ResponseBean response = new ResponseBean();
        IPage<TaCustomer> pg = new Page<>(pageNum, pageSize);
        IPage<TaCustomer> result = taCustomerMapper.customerPage(pg,customerName,phone,platNumber);
        response.addSuccess(result);
        return response;
    }

    @Override
    public boolean newRecordByMiniapp(TaMainUsherRecord taMainUsherRecord) throws Exception {
        Integer consultantId = taMainUsherRecord.getConsultantId();
        if (null == consultantId || consultantId < 1) {
            throw new Exception("置业顾问不能为空");
        }

        if (StringUtils.isEmpty(taMainUsherRecord.getPhone())) {
            throw new Exception("客户电话不能为空");
        }

        taMainUsherRecord.setStatus(StatusUtils.Normal);

        // 新增客户
        iTaCustomerService.newCustByMainUsher(taMainUsherRecord);

        return taMainUsherRecordMapper.insert(taMainUsherRecord) > 0;
    }

    @Override
    public boolean updateByMiniapp(TaMainUsherRecord taMainUsherRecord) throws Exception {
        Integer consultantId = taMainUsherRecord.getConsultantId();
        if (null == consultantId || consultantId < 1) {
            throw new Exception("置业顾问不能为空");
        }

        if (StringUtils.isEmpty(taMainUsherRecord.getPhone())) {
            throw new Exception("客户电话不能为空");
        }

        taMainUsherRecord.setStatus(StatusUtils.Normal);

        int row = taMainUsherRecordMapper.updateById(taMainUsherRecord);

        // 是否更新了置业
        TaCustomer taCustomer = iTaCustomerService.getById(taMainUsherRecord.getCustomerId());
        Integer originConsultant = taCustomer.getConsultantId();

        // 是否需要更新客户信息
        if (StringUtils.isEmpty(taCustomer.getCustomerName()) || StringUtils.isEmpty(taCustomer.getPhone())) {
            if (StringUtils.isEmpty(taCustomer.getCustomerName())) {
                taCustomer.setCustomerName(taMainUsherRecord.getPersonName());
            }

            if (StringUtils.isEmpty(taCustomer.getPhone())) {
                taCustomer.setPhone(taMainUsherRecord.getPhone());
            }

            iTaCustomerService.updateById(taCustomer);
        }

        // 如果原来置业为空
        if (null == originConsultant || originConsultant < 1) {
            iTaCustomerService.updateConsultant(taCustomer, consultantId);
        } else if (!consultantId.equals(originConsultant)) {
            // 如果临时替换跟踪置业
            // 那么需要给原始置业, 以及临时置业各自发一条消息
            String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + String.valueOf(taCustomer.getCustomerId());
            String tplId = miniApp.getTpls().getVisitor();

            // 原绑定置业
            String rcOpenid = taWechatUserMapper.getOpenidOf(originConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(rcOpenid)) {
                throw new Exception("置业尚未绑定微信小程序");
            }
            SysUser rcUser = sysUserMapper.selectById(originConsultant);

            // 临时置业
            String proxyOpenid = taWechatUserMapper.getOpenidOf(consultantId, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(proxyOpenid)) {
                throw new Exception("置业尚未绑定微信小程序");
            }
            SysUser proxyUser = sysUserMapper.selectById(consultantId);

            // 通知原置业, 客户被临时置业接待
            List<Object> dt1 = new ArrayList<Object>(){{
                add(taCustomer.getCustomerName());
                add(taCustomer.getPhone());
                add("您有新客户来访, 已被安排给 " + proxyUser.getUsername() + " 接待!");
            }};
            miniApp.sendTPLMessage(tplId, rcOpenid, page, dt1);

            // 通知临时置业
            List<Object> dt2 = new ArrayList<Object>(){{
                add(taCustomer.getCustomerName());
                add(taCustomer.getPhone());
                add(rcUser.getUsername() + " 有新客户来访, 请协助接待一下!");
            }};
            miniApp.sendTPLMessage(tplId, proxyOpenid, page, dt2);
        }

        return row > 0;
    }

    @Override
    public boolean deleteRecord(Integer id) {
        UpdateWrapper<TaMainUsherRecord> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Delete);
        updateWrapper.eq("rec_id", id);
        int row = taMainUsherRecordMapper.update(new TaMainUsherRecord(), updateWrapper);

        return row > 0;
    }
}
