package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.TaVisitorAppointmentMapper;
import com.huiju.welcome.model.TaVisitorAppointment;
import com.huiju.welcome.service.ITaVisitorAppointmentService;
import com.huiju.welcome.utils.DateUtils;
import com.huiju.welcome.utils.StatusUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@Service
public class TaVisitorAppointmentServiceImpl extends ServiceImpl<TaVisitorAppointmentMapper, TaVisitorAppointment> implements ITaVisitorAppointmentService {
    @Autowired
    TaVisitorAppointmentMapper taVisitorAppointmentMapper;

    @Override
    public ResponseBean check(TaVisitorAppointment taVisitorAppointment) {
        String phone = taVisitorAppointment.getPhone();
        String platNumber = taVisitorAppointment.getPlatNumber();
        LocalDateTime visitDate = taVisitorAppointment.getVisitDate();

        List<TaVisitorAppointment> list = taVisitorAppointmentMapper.getSameAppointments(platNumber, phone, DateUtils.localDatetime2day(visitDate));
        if (null != list && list.size() > 0) {
            return ResponseBean.error("请勿对该人员进行重复预约", HttpStatus.SC_BAD_REQUEST);
        }

        return ResponseBean.success("");
    }

    @Override
    public List<TaVisitorAppointment> getAppointmentsBy(Map<String, Object> filters) {
        QueryWrapper<TaVisitorAppointment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", filters.get("phone"));
        queryWrapper.eq("status", StatusUtils.Normal);
        queryWrapper.orderByDesc("create_date");

        return taVisitorAppointmentMapper.selectList(queryWrapper);
    }

    @Override
    public List<TaVisitorAppointment> getAppointmentsByPlateNumber(String plateNumber) {
        QueryWrapper<TaVisitorAppointment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plat_number", plateNumber);
        queryWrapper.eq("status", StatusUtils.Normal);
        queryWrapper.orderByDesc("create_date");

        return taVisitorAppointmentMapper.selectList(queryWrapper);
    }

    @Override
    public ResponseBean visitorAppointmentList(IPage pg, String platNumber, String personName,String phone) {
       ResponseBean responseBean= new ResponseBean();
        IPage<TaVisitorAppointment> list= taVisitorAppointmentMapper.visitorAppointmentList(pg,platNumber,personName,phone);
        responseBean.addSuccess(list);
        return responseBean;
    }
}
